<?php
abstract class APIBaseTest extends PHPUnit_Framework_TestCase
{

    public $hostA_id = "305658693b94e003e765956f1609731419cbc0e5c9caa09e230df5e005f1f283";
    public $hostA_ip = "10.0.0.150";
    public $hostB_id = "33736d45041e2a9407be8cf449aeffa95114bef661c20deaca1bbcfbc2922856";
    public $hostB_ip = "10.0.0.153";
    public $ldapSettings = '{
                "ldapEnabled": true
                }';
    public $admin_access_token = '';
    /*NOTE: For ldap server that have anonomyous ldap search access, ldapusername and password can be empty. For ldapserver that
          need aurthentication to happen for doing ldaop search ldapUsername and ldapPassword are required. Our test ldaps server have
          anonomyous ldap serach access, meaning , ldapusers are search able(browseable) without being authenticated*/

    public function __construct()
    {
        parent::__construct();

        $this->pg_cfdb = pg_connect("host=localhost dbname=cfdb port=5444");
        $this->pg_cfsettings = pg_connect("host=localhost dbname=cfsettings port=5444");
    }

    function assertValidJson($data)
    {
        if ($data === null && json_last_error() !== JSON_ERROR_NONE)
        {
            $this->fail("Bad json input string");
        }
        return $data;
    }

    function array_keys_exists($array, $keys)
    {


        foreach ($keys as $k)
        {
            if (!isset($array[$k]))
            {
                return false;
            }
        }
        return true;
    }

    public function setUp()
    {
        $this->loadFixturesSQL();
        $this->baseUrl = "http://localhost:8888/api";
        $this->pest = null;
        $this->pest = new PestNova($this->baseUrl);
        if (!OAUTH2ENABLED)
        {
            $this->pest->setupAuth("admin", "admin");
        }
        else
        {
            $this->setUpAuthAuthentication();
        }


    }

    public function setUpAuthAuthentication()
    {
        $data = array(
            'username' => 'admin',
            'password' => 'admin',
            'client_id' => 'APITEST',
            'grant_type' => 'password',
            'client_secret' => 'APITEST'
        );

        $data = $this->pest->post('/oauth2/token', json_encode($data));
        if (is_array($data))
        {
            if (!empty($data['access_token']))
            {
                $this->pest->setupOauthAuth($data['access_token']);
            }
            else
            {
                throw new Exception('Invalid server response cannot access server token.');
            }
        }
    }

    public function tearDown()
    {
        pg_query($this->pg_cfdb, "TRUNCATE __Hosts");
        pg_query($this->pg_cfdb, "TRUNCATE __Contexts");
        pg_query($this->pg_cfdb, "TRUNCATE __Variables");
        pg_query($this->pg_cfdb, "TRUNCATE __LastSeenHosts");
        pg_query($this->pg_cfdb, "TRUNCATE __MonitoringMg");

        pg_query($this->pg_cfsettings, "TRUNCATE ScheduledReports");
        pg_query($this->pg_cfsettings, "TRUNCATE Settings");
        pg_query($this->pg_cfsettings, "TRUNCATE oauth_clients");
        pg_query($this->pg_cfsettings, "TRUNCATE Users");
        pg_query($this->pg_cfsettings, "TRUNCATE Roles");
    }

    public function getResults($path)
    {
        $jsonArray = $this->pest->get($path);
        $this->assertValidJson($jsonArray);
        return $jsonArray['data'];
    }

    public function addHeader($header)
    {
        $this->pest->curl_opts[CURLOPT_HTTPHEADER][] = $header;
    }

    public function loadFixturesSQL()
    {
        $fixtures = array(
            'hosts' => 'cfdb',
            'contexts' => 'cfdb',
            'variables' => 'cfdb',
            'lastseenhosts' => 'cfdb',
            #'monitoringmg' => 'cfdb',

            'settings' => 'cfsettings',
            'users' => 'cfsettings',
            'roles' => 'cfsettings',
            'oauth_clients' => 'cfsettings'
        );


        foreach ($fixtures as $table => $db)
        {
            $filename = __DIR__ . "/fixtures/" . $db . "." . $table . ".sql";
            $stmts = file_get_contents($filename);

            if ($db == 'cfdb')
            {
                pg_query($this->pg_cfdb, $stmts);
            }
            else if ($db == 'cfsettings')
            {
                pg_query($this->pg_cfsettings, $stmts);
            }
            else
            {
                die();
            }
        }
    }
}
