<?php

/**
 * Class ProfileModel
 */
class ProfileModel extends Cf_Model
{
    const USER_URL = '/user/%s';

    /**
     * @param $username
     * @return array
     */
    public function getProfileData($username)
    {
        $profileData = json_decode(
            $this->getRestClient()->get(sprintf(self::USER_URL, $username)),
            JSON_OBJECT_AS_ARRAY
        );
        return isset($profileData['data']) ? $profileData['data'][0] : [];
    }

    /**
     * @return array
     */
    public function getTimezoneList()
    {
        //get time zones from postgres
        $timeZones = $this->db->query('SELECT * FROM pg_timezone_names WHERE name LIKE \'%/%\' ORDER BY name')->result_array();
        $result = [];
        foreach ($timeZones as $tz) {
            list($continent, $city) = explode('/', $tz['name']);

            $utcOffset = $this->convertUtcOffset($tz['utc_offset']);
            $result[$continent][] = [
                'label' => $city . " (GMT $utcOffset)",
                'value' => $tz['name']
            ];
        }
        return $result;
    }

    /**
     * @param $name
     * @return array
     */
    public function getTimeZone($name)
    {
        $timeZone = $this->db->like('name', $name)->get('pg_timezone_names')->row_array();

        $utcOffset = $this->convertUtcOffset($timeZone['utc_offset']);
        $result = [
            'label' => $timeZone['name'] . " (GMT $utcOffset)",
            'value' => $timeZone['name'],
            'offset' => $utcOffset,
            'minutesOffset' => $this->convertOffsetToMinutes($utcOffset)
        ];

        return $result;
    }

    /**
     * convert offset from `01:00:00` to `+01:00` or `-02:00:00` to `-02:00`
     */
    private function convertUtcOffset($offset)
    {
        return $offset[0] == '-' ? substr($offset, 0, 6) : '+' . substr($offset, 0, 5);
    }

    private function convertOffsetToMinutes($offset)
    {
       $sign = $offset[0];
       list($hours, $minutes) = explode(':', substr($offset, 1));
       $offsetInMinutes = $sign . (intval($hours) * 60 + intval($minutes));
       return intval($offsetInMinutes);
    }

    /**
     * @param $data
     * @param $username
     * @return mixed
     */
    public function updateProfile($data, $username)
    {
        $this->getRestClient()->post(sprintf(self::USER_URL, $username), $data);
        return true;
    }

    /**
     * @param $value
     * @param $username
     */
    public function setNeverAskTimeZoneChange($value, $username)
    {
        $this->db->where('username', $username)->update('users', ['never_ask_timezone_change' => $value]);
    }

    /**
     * @param $value
     * @param $username
     */
    public function setUseBrowserTime($value, $username)
    {
        $this->db->where('username', $username)->update('users', ['use_browser_time' => $value]);
    }

    /**
     * @param $username
     * @return int
     */
    public function getNeverAskTimeZoneChange($username)
    {
        $value = $this->db
            ->where('username', $username)
            ->select('never_ask_timezone_change')
            ->get('users')
            ->row_array();
        return isset($value['never_ask_timezone_change']) ? $value['never_ask_timezone_change'] : 0;
    }

    public function getCurrentTimeZone($username)
    {
        $profile = $this->getProfileData($username);
        $tz = isset($profile['time_zone']) ? $this->getTimeZone($profile['time_zone']) : null;
        return $tz;
    }

    /**
     * @param $username
     * @return int
     */
    public function getUseBrowserTime($username)
    {
        $value = $this->db
            ->where('username', $username)
            ->select('use_browser_time')
            ->get('users')
            ->row_array();
        return isset($value['use_browser_time']) ? $value['use_browser_time'] : 0;
    }

}
