<?php
require_once 'AlertSQL.php';

class InventoryAlert extends AlertSQL
{


    function __construct()
    {
        parent::__construct();
    }


    /**
     * @param $alert
     * @param $rule
     * @param $limit
     * @param Pest $restClient
     * @return array
     */
    function getInventoryHostListWithHeader($alert, $rule, $limit, Pest $restClient)
    {

        $includes = $this->utils_getIncludesFromAlert($alert);
        $excludes = $this->utils_getExcludesFromAlert($alert);
        $dataToApi = [
            'select' => $this->getSelectForInventory($rule->inventoryConditions["columns"]),
            'filter' => $this->getFilterForInventory($rule->inventoryConditions["filters"]),
            'limit' => $limit,
            'hostContextInclude' => $includes,
            'hostContextExclude' => $excludes,
        ];

        $response = $restClient->post('/inventory', $dataToApi);
        $response = json_decode($response, JSON_OBJECT_AS_ARRAY);

        $result = array();
        $result['header'] = $response["data"][0]['header'];
        $result['rows'] = $response["data"][0]['rows'];
        return $result;
    }


    /**
     * @param $alert
     * @param $rule
     * @param Pest $restClient
     * @return int
     */
    function getCountFailHosts($alert, $rule, Pest $restClient)
    {
        $includes = $this->utils_getIncludesFromAlert($alert);
        $excludes = $this->utils_getExcludesFromAlert($alert);
        $dataToApi = [
            'select' => ['resultCount'],
            'filter' => $this->getFilterForInventory($rule->inventoryConditions["filters"]),
            'limit' => 99999999,
            'hostContextInclude' => $includes,
            'hostContextExclude' => $excludes,
        ];

        $response = $restClient->post('/inventory', $dataToApi);
        $response = json_decode($response, JSON_OBJECT_AS_ARRAY);

        return (int) reset($response["data"])['rows'][0][0];
    }

    /**
     * Return SQL string for inventory rule. Use this function to calculate alert stat MUST include hostkey only
     *
     * @param <object> $rule
     * @return <string>
     */
    function getStatSqlStringForInventory($rule)
    {
        // SQL espression should contain only 1 column in select
        // for example: SELECT DISTINCT Hosts.HostKey  FROM Hosts INNER JOIN INVENTORY ON INVENTORY.HostKey = Hosts.HostKey  LEFT JOIN inventory table_default_sys_arch ON Hosts.HostKey = table_default_sys_arch.HostKey AND table_default_sys_arch.keyname = 'default.sys.arch' WHERE table_default_sys_arch.value ILIKE '%6%' GROUP BY Hosts.hostkey , table_default_sys_arch.value
        // so we will remove everything until "FROM Hosts" and replace with SELECT DISTINCT  hosts.hostkey


        $SQL = $rule->inventoryConditions['SQLSTRING'];

        // find SELECT and insert hostkey as first field
        $pos = stripos($SQL, 'FROM Hosts');

        if ($pos === false) {
            // something is wrong, return as it is
            return $SQL;
        }

        // we found SELECT - replace with SELECT hostkey
        $tmp = substr($SQL, $pos); // 15- strlen of SELECT DISTINCT

        $SQL = "SELECT DISTINCT hosts.hostkey " . $tmp;

        return $SQL;
    }

    function getStatForInventory($filters, $includeGroup, $excludeGroup, Pest $restClient)
    {
        $dataToApi = [
            'select' => ['hostkey'],
            'filter' => $this->getFilterForInventory($filters),
            'limit' => 99999999,
            'hostContextInclude' => $includeGroup,
            'hostContextExclude' => $excludeGroup,
        ];

        $response = $restClient->post('/inventory', $dataToApi);
        $response = json_decode($response);

        $hosts = array_map(function ($item) {
            return $item[0];
        }, $response->data[0]->rows);

        return $hosts;
    }

    private function getFilterForInventory($filters)
    {
        $operators = [
            '<' => '<',
            '>' => '>',
            '=' => '=',
            '!=' => '!=',
            '<=' => '<=',
            '>=' => '>=',
            'ILIKE' => 'matches',
            'NOT ILIKE' => 'not_match',
            'matches' => 'matches',
            'not_match' => 'not_match',
            'contains' => 'contains',
            'not_contain' => 'not_contain',
        ];

        $outputFilter = [];
        foreach ($filters as $filter) {
            $outputFilter[$filter['label']][$operators[$filter['condition']]] = $this->getWrappedValue($filter['condition'],
                $filter['value']);
        }
        return $outputFilter;
    }

    private function getSelectForInventory($columns)
    {
        $output = [];
        foreach ($columns as $column) {
            $output[] = $column['label'];
        }
        return $output;
    }

    private function getWrappedValue($operator, $value)
    {
        $wraper = ['ILIKE' => '%', 'LIKE' => '%'];

        if (key_exists($operator, $wraper)) {
            return $wraper[$operator] . $value . $wraper[$operator];
        } else {
            return $value;
        }

    }

}