/*jslint node: true */
/*jslint browser: true*/
/*jslint nomen: true*/
/*jslint plusplus: true */
/*global dashboardApp, angular, $*/

'use strict';

dashboardApp.factory('changesWidgetHelper',
    ['$q', 'widgetHelper', '$window', '$location',
        function ($q, widgetHelper, $window, $location) {
            var helper = {};

            helper.goToChangesReport = function (daterange) {
                sessionStorage.clear();
                var url = MP_SITE_URL + "/advancedreports#/changes";
                if (daterange !== undefined || angular.equals({}, daterange)) {
                    url += "?from=" + daterange.from + "&to=" + daterange.to;
                }
                $window.location.href = url;

            }

            helper.chartConfig = {
                options: {
                    chart: {
                        type: 'column',
                        backgroundColor: 'rgba(255, 255, 255, 255)',
                        height: '194',
                        width: '420',
                        events: {
                            click: function (e) {
                                helper.goToChangesReport()
                            }
                        }
                    },
                    colors: ['#0071a6'],
                    exporting: {
                        enabled: false
                    },
                    plotOptions: {
                        column: {
                            allowPointSelect: false,
                            cursor: 'pointer',
                            point: {
                                events: {
                                    click: function () {
                                        var daterange = {};
                                        daterange.from = moment().startOf('day').subtract((6 - this.x), 'days');
                                        daterange.to = moment().startOf('day').subtract((5 - this.x), 'days').subtract('1', 'milliseconds'); // remove a ms to keep within day
                                        helper.goToChangesReport(daterange);
                                    }
                                }
                            }
                        }
                    },
                    tooltip: {
                        formatter: function () {
                            return 'Changes: ' + this.y;
                        }
                    },
                    legend: {
                        enabled: false
                    }
                },
                series: [{
                    data: []
                }],
                title: {
                    text: ''
                },
                subtitle: {
                    text: ''
                },
                xAxis: {
                    categories: [],
                    title: {
                        text: null
                    },
                    allowDecimals: false
                },
                yAxis: {
                    allowDecimals: false,
                    min: 0,
                    title: {
                        text: 'Changes by CFEngine'
                    }
                },
                credits: {
                    enabled: false
                },
                loading: false
            };


            helper.getChangeStats = function (data) {

                var promise = widgetHelper.getChangesStats(data).then(
                    function (result) {
                        return result;
                    },
                    function (error) {
                        // error
                    }
                );

              return  $q.all(promise)
            };


            helper.convertArray = function (arr) {
                var returnData = {};
                returnData.array = [];
                returnData.axis = [];

                angular.forEach(arr, function (value, key) {
                    returnData.axis.push(value.date);
                    returnData.array.push(value.count);
                });

                return returnData;
            };

            helper.setChartData = function (changeStats) {

                var convertedData = helper.convertArray(changeStats);
                helper.chartConfig.series[0] = {
                    data: convertedData.array
                };
                helper.chartConfig.xAxis.categories = convertedData.axis;

            };


            return helper;
        }]);