$('html').on('host-page-loaded', function () {
    let hostkey = $('.host_id_SE').text().replace('SHA=', '').trim();
    let updateInventoryValue = function (hostkey, attribute) {
        let data = {
            "select": [attribute],
            "start": 0,
            "limit": 1,
            "showSQL": false,
            "hostContextInclude": [hostkey],
            "hostContextExclude": []
        };
        $.ajax({
            type: "POST",
            url: '/advancedreports/inventory/data',
            dataType: 'json',
            data: JSON.stringify(data)
        }).done(function (response) {
            let value = (response.rows != undefined && response.rows.length > 0) ? response.rows[0][0] : 'n/a';
            $('#host-inventory-data').html(value);
            processInventoryPin(attribute);
            $('.inventory-report-btn').data('attribute', attribute).data('hostkey', hostkey)
        });
    }

    $.ajax({
        type: "GET",
        url: '/advancedreports/inventory/variables',
        dataType: 'json'
    }).done(function (response) {
        let result = {}
        let isValueUpdated = false;
        response.forEach(function (attr) {
            if (!isValueUpdated) {
                updateInventoryValue(hostkey, attr.attribute_name);
                isValueUpdated = true;
            }
            if (result[attr.category] == undefined) {
                result[attr.category] = [attr.attribute_name]
            } else {
                result[attr.category].push(attr.attribute_name)
            }
        })

        let selectHtml = '';
        Object.keys(result).forEach(function (group) {
            selectHtml += '<optgroup label="' + group + '">';
            result[group].forEach(function (attributeName) {
                selectHtml += '<option value="' + attributeName + '">' + attributeName + '</option>';
            });
            selectHtml += '</optgroup>';
        })
        $('.chosen-select').append(selectHtml);

        $(".chosen-select").chosen({disable_search_threshold: 10}).change(function (event, data) {
            updateInventoryValue(hostkey, data.selected);
        });
    });

    $('.inventory-report-btn').click(function () {
        let el = $(this);
        window.open(
            encodeURI(el.data('url') + ',' + el.data('attribute') + '&include=' + el.data('hostkey').trim()),
            '_blank'
        );
    });

    let openModal = function (event) {
        var userModalBody = {
            'header': '<h2>' + event.data.type.capitalizeFirstLetter() + ' defined on ' + $('.host_alias').text().trim().htmlSpecialChars() + '</h2>',
            'body_url': MP_SITE_URL + '/advancedreports/renderTable/' + event.data.type + '/' + $('.host_id_SE').text().trim()
        };
        var mainDialog = bootbox.dialog('', [], userModalBody).css({
            'width': function () {
                return ($(document).width() * 0.6) + 'px';
            },
            'margin-left': function () {
                return -($(this).width() / 2);
            }
        });
        mainDialog.find('.modal-body').addClass('form-buttons').css('height', ($(document).height() * 0.4) + 'px');
    };

    $('.show-classes').click({type: 'classes'}, openModal);
    $('.show-variables').click({type: 'variables'}, openModal);

    $('body').on("pinned_loaded", function () {
        $("#pinned_items_wrapper").pinnedItems({data: pinnedItems});
        processInventoryPin($('i[pin-type="inventory"]').attr('pin-name'));
    });

    loadPinnedItems();
    replacePinElements();
});

let pinnedItems = [];

const pinItem = function (element) {
    $.post(MP_SITE_URL + "/pinItem/pin", {
        data: {type: element.getAttribute('pin-type'), name: element.getAttribute('pin-name')}
    }, function (data) {
        loadPinnedItems();
        $.notify('Pinned', {
            className: "success"
        });
        $(element).replaceWith(
            buildPinElement(true, element.getAttribute('pin-name'), element.getAttribute('pin-type'), data.id)
        );
    }, "json");
}

function unPinItem(element) {
    $.ajax({
        type: "DELETE",
        url: MP_SITE_URL + "/pinItem/pin/" + element.getAttribute('pin-id')
    }).done(function (response) {
        loadPinnedItems();
        $.notify('Unpinned', {
            className: "success"
        });

        $(element).replaceWith(
            buildPinElement(false, element.getAttribute('pin-name'), element.getAttribute('pin-type'))
        );
    });
}

function loadPinnedItems() {
    $.get(encodeURI(MP_SITE_URL + "/pinItem/pin?hostkey=" + $('.host_id_SE').text().trim()))
        .then(function (data) {
            pinnedItems = JSON.parse(data);
            $('body').trigger("pinned_loaded");
        });
}

function replacePinElements() {
    $('.create-pin').each(function (index, element) {
        const name = $(element).attr('pin-name');
        const type = $(element).attr('pin-type');

        // if item pinned then show unpin element
        let pinnedItem = pinnedItems[type] != undefined ? pinnedItems[type].filter(item => item.name == name) : [];
        if (pinnedItem.length > 0) {
            $(element).replaceWith(buildPinElement(true, name, type, pinnedItem[0].id));
        } else {
            $(element).replaceWith(buildPinElement(false, name, type));
        }
    });
}

function buildPinElement(pinned, name, type, id = null) {
    let element;
    if (pinned === true) {
        element = `<i title="Click to remove from host info page"  
                      class="icon-pushpin green unpin" 
                      onclick="unPinItem(this)"  
                      pin-id="${id}"  
                      pin-name="${name}" 
                      pin-type="${type}"> 
                   </i>`;

    } else {
        element = `<i title="Click to pin to host info page" 
                      class="icon-pushpin muted pin" 
                      onclick="pinItem(this)"  
                      pin-name="${name}" 
                      pin-type="${type}" >
                   </i>`;
    }
    return element;
}

function processInventoryPin(attribute) {
    $('.icon-pushpin[pin-type="inventory"]').replaceWith(`
                <i class="create-pin" pin-type="inventory" pin-name="${attribute}"></i>
            `);
    replacePinElements();
}

$.widget('ui.pinnedItems', {
    options: {
        data: {}
    },
    templates: {
        main: '<h4><i class="icon-pin"></i>Pinned items</h4> <div class="pinned-container">%body%</div>',
        group: '<div> <h5>%group_name%</h5> %items% </div>',
        item: `<div class="row-fluid break-all"> 
                 <span class="muted span5">%item_name%:</span> 
                 <span class="span7 break-all">
                 %item_value%
                 <i title="Click to remove from host info page" 
                    class="icon-pushpin green unpin" 
                    onclick="unPinItem(this)" 
                    pin-id="%item_id%" 
                    pin-name="%item_name%" 
                    pin-type="%item_type%">
                 </i>
                 </span> 
              </div>`,
    },

    _init: function () {
        this.element.html(this._buildHtml(this.options.data));
    },

    _getGroupName: function (name) {
        switch (name) {
            case 'class':
                name = 'Classes';
                break;
            case 'variable':
                name = 'Variables';
                break;
            default:
                name = name.capitalizeFirstLetter()
        }
        return name;
    },

    _buildHtml: function (data) {
        let self = this;
        let html = this.templates.main;
        let groupsHtml = '';
        if (data !== undefined && Object.keys(data).length > 0) {
            Object.keys(data).forEach(function (group) {
                let groupHtml = self.templates.group.replace('%group_name%', self._getGroupName(group));
                let itemsHtml = '';
                data[group].forEach(function (item) {
                    let value = group == 'class' ? item.value == true ? 'defined' : 'not defined' : item.value;
                    itemsHtml += self.templates.item
                        .replace('%item_name%', item.name)
                        .replace('%item_value%', value)
                        .replace('%item_id%', item.id);
                })
                groupHtml = groupHtml.replace('%items%', itemsHtml);
                groupsHtml += groupHtml;
            });
            return html.replace('%body%', groupsHtml);
        } else {
            return '';
        }
    }
});
