\set ON_ERROR_STOP true

-------------------------------------------------------------------------------
-- Create settings database:
-------------------------------------------------------------------------------

CREATE TABLE IF NOT EXISTS ScheduledReports (
	username		TEXT		NOT NULL,
	query			TEXT		NOT NULL,
	query_id		TEXT		NOT NULL,
	run_classes		TEXT		NOT NULL,
	last_executed 		TEXT,
	email			TEXT,
	email_title		TEXT,
	email_description	TEXT,
	host_include		TEXT[],
	host_exclude		TEXT[],
        already_run		BOOLEAN		DEFAULT FALSE,
        enabled			BOOLEAN		DEFAULT TRUE,
	output			TEXT[]		NOT NULL
    );

ALTER TABLE ScheduledReports
ADD COLUMN IF NOT EXISTS excludedhosts json DEFAULT NULL;

CREATE TABLE IF NOT EXISTS Settings (
        key                     TEXT		NOT NULL,
        value			JSON
    );
CREATE UNIQUE INDEX IF NOT EXISTS settings_unique_key_idx ON settings (key);

CREATE TABLE IF NOT EXISTS Users (
        username                TEXT		NOT NULL,
        password                TEXT,
        salt                    TEXT,
        name                    TEXT,
        email                   TEXT,
        external                BOOLEAN         DEFAULT FALSE,
        active                  BOOLEAN         DEFAULT FALSE,
        roles                   TEXT[]          DEFAULT '{}',
        time_zone               TEXT            DEFAULT 'Etc/GMT+0',
        ChangeTimestamp         timestamp with time zone       DEFAULT now()
    );

-- Add time_zone column to users --
-- Default value is Etc/GMT+0 what means no time offset (GMT) by default if time_zone is not specified --
ALTER TABLE Users ADD COLUMN IF NOT EXISTS time_zone TEXT DEFAULT 'Etc/GMT+0';

CREATE UNIQUE INDEX IF NOT EXISTS users_uniq ON Users (username, external);

CREATE TABLE IF NOT EXISTS Roles (
        name                    TEXT		NOT NULL,
        description             TEXT,
        include_rx              TEXT,
        exclude_rx              TEXT,
        ChangeTimestamp         timestamp with time zone       DEFAULT now()
    );

ALTER TABLE Roles ADD COLUMN IF NOT EXISTS is_default boolean NULL DEFAULT 'false';

-- 3.12.1 drop sketches
ALTER TABLE Roles DROP COLUMN IF EXISTS sketches;

DROP INDEX IF EXISTS roles_uniq CASCADE;
CREATE UNIQUE INDEX IF NOT EXISTS roles_uniq ON Roles (name);

CREATE TABLE IF NOT EXISTS LicenseInfo (
        ExpireTimeStamp         timestamp with time zone    NOT NULL,
        InstallTimestamp        timestamp with time zone,
        Organization            TEXT,
        LicenseType             TEXT,
        LicenseCount            integer
    );

CREATE TABLE IF NOT EXISTS KeysPendingForDeletion (
        HostKey     TEXT
    );


 -- FOR OAUTH

CREATE TABLE IF NOT EXISTS  oauth_clients (client_id VARCHAR(80) NOT NULL, client_secret VARCHAR(80) NOT NULL, redirect_uri VARCHAR(2000) NOT NULL, grant_types VARCHAR(80), scope VARCHAR(100), user_id VARCHAR(80), CONSTRAINT client_id_pk PRIMARY KEY (client_id));
CREATE TABLE IF NOT EXISTS  oauth_access_tokens (access_token VARCHAR(40) NOT NULL, client_id VARCHAR(80) NOT NULL, user_id VARCHAR(255), expires TIMESTAMP NOT NULL, scope VARCHAR(2000), CONSTRAINT access_token_pk PRIMARY KEY (access_token));
CREATE TABLE IF NOT EXISTS  oauth_authorization_codes (authorization_code VARCHAR(40) NOT NULL, client_id VARCHAR(80) NOT NULL, user_id VARCHAR(255), redirect_uri VARCHAR(2000), expires TIMESTAMP NOT NULL, scope VARCHAR(2000), CONSTRAINT auth_code_pk PRIMARY KEY (authorization_code));
CREATE TABLE IF NOT EXISTS  oauth_refresh_tokens (refresh_token VARCHAR(40) NOT NULL, client_id VARCHAR(80) NOT NULL, user_id VARCHAR(255), expires TIMESTAMP NOT NULL, scope VARCHAR(2000), CONSTRAINT refresh_token_pk PRIMARY KEY (refresh_token));
--CREATE TABLE IF NOT EXISTS  oauth_users (username VARCHAR(255) NOT NULL, password VARCHAR(2000), first_name VARCHAR(255), last_name VARCHAR(255), CONSTRAINT username_pk PRIMARY KEY (username));
CREATE TABLE IF NOT EXISTS  oauth_scopes (scope TEXT, is_default BOOLEAN);
CREATE TABLE IF NOT EXISTS  oauth_jwt (client_id VARCHAR(80) NOT NULL, subject VARCHAR(80), public_key VARCHAR(2000), CONSTRAINT client_id_pk_jwt PRIMARY KEY (client_id));




CREATE OR REPLACE FUNCTION delete_expired_access() RETURNS TRIGGER AS $_$
BEGIN
    DELETE FROM oauth_access_tokens WHERE expires <  now() AT TIME ZONE 'UTC';
    DELETE FROM oauth_refresh_tokens WHERE expires < now() AT TIME ZONE 'UTC';
    RETURN NULL;
END $_$ LANGUAGE 'plpgsql';

DROP TRIGGER IF EXISTS delete_expired_access_token ON oauth_access_tokens;
CREATE TRIGGER delete_expired_access_token AFTER INSERT ON oauth_access_tokens FOR EACH ROW EXECUTE PROCEDURE delete_expired_access();

CREATE TABLE IF NOT EXISTS external_roles_map (
        external_role                    TEXT		NOT NULL,
        internal_role                TEXT,
        ChangeTimestamp         timestamp with time zone       DEFAULT now()
    );

CREATE UNIQUE INDEX IF NOT EXISTS external_roles_uniq ON external_roles_map (external_role);


CREATE TABLE IF NOT EXISTS rbac_permissions (
    "alias" character varying(100) PRIMARY KEY,
    "group" character varying(50),
    "name" character varying(100),
    "description" character varying(200),
    "application" character varying(50),
    "allowed_by_default" boolean DEFAULT false NOT NULL
);
COMMENT ON TABLE rbac_permissions IS 'This table stores RBAC permissions';

CREATE TABLE IF NOT EXISTS rbac_role_permission (
    "role_id" character varying NOT NULL,
    "permission_alias" character varying NOT NULL,
    CONSTRAINT "rbac_role_permission_role_id_permission_alias" UNIQUE ("role_id", "permission_alias"),
    CONSTRAINT "rbac_role_permission_permission_alias_fkey" FOREIGN KEY (permission_alias) REFERENCES rbac_permissions(alias) ON UPDATE CASCADE ON DELETE CASCADE,
    CONSTRAINT "rbac_role_permission_role_id_fkey" FOREIGN KEY (role_id) REFERENCES roles(name) ON UPDATE CASCADE ON DELETE CASCADE
);
COMMENT ON TABLE rbac_role_permission IS 'This table associates roles to permissions in a 1-to-many relationship';

CREATE OR REPLACE FUNCTION set_default_permissions() RETURNS TRIGGER AS $_$
BEGIN
    INSERT INTO rbac_role_permission (permission_alias, role_id) (
    SELECT alias as permission_alias, NEW.name::text as role_id
    FROM rbac_permissions
    WHERE allowed_by_default = true
    );
    RETURN NULL;
END $_$ LANGUAGE 'plpgsql';
COMMENT ON FUNCTION set_default_permissions IS 'This function assigns allowed by default permissions to new roles';

DROP TRIGGER IF EXISTS set_default_permissions_trigger ON roles;
CREATE TRIGGER set_default_permissions_trigger AFTER INSERT ON roles FOR EACH ROW EXECUTE PROCEDURE set_default_permissions();

CREATE OR REPLACE FUNCTION insert_default_permission() RETURNS TRIGGER AS $_$
DECLARE
    role record;
BEGIN
    FOR role IN
        EXECUTE 'SELECT name FROM roles WHERE name NOT IN (''admin'', ''cf_remoteagent'')'
    LOOP
        IF NEW.allowed_by_default  = TRUE THEN
            EXECUTE 'INSERT INTO rbac_role_permission (permission_alias, role_id) ' ||
                    'VALUES ($1, $2) ' ||
                    'ON CONFLICT (role_id, permission_alias) DO NOTHING' USING NEW.alias, role.name;
        END IF;
    END LOOP;
    RETURN NULL;
END $_$ LANGUAGE 'plpgsql';
COMMENT ON FUNCTION insert_default_permission IS 'This function assigns allowed by default permissions to existing roles';

DROP TRIGGER IF EXISTS insert_default_permission_trigger ON rbac_permissions;
CREATE TRIGGER insert_default_permission_trigger AFTER INSERT OR UPDATE ON rbac_permissions FOR EACH ROW EXECUTE PROCEDURE insert_default_permission();


CREATE TABLE IF NOT EXISTS remote_hubs (
    id bigserial  PRIMARY KEY,
    hostkey text,
    ui_name character varying(70) UNIQUE,
    api_url text NULL,
    target_state character varying(20) NOT NULL,
    transport json NOT NULL,
    role character varying(50) NOT NULL
);

COMMENT ON TABLE remote_hubs IS 'This table stores federated reporting remote hub';


CREATE TABLE  IF NOT EXISTS federated_reporting_settings (
    "key" character varying DEFAULT 'false' PRIMARY KEY,
    "value" text
);

COMMENT ON TABLE federated_reporting_settings IS 'This table stores federated reporting settings';
